package com.rjconsultores.ventaboletos.dao.hibernate;


import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.AliasClasseDAO;
import com.rjconsultores.ventaboletos.entidad.AliasClasse;

@Repository("aliasClasseDAO")
@SuppressWarnings("unchecked")
public class AliasClasseHibernateDAO extends GenericHibernateDAO<AliasClasse, Integer>
		implements AliasClasseDAO {

	@Autowired
	public AliasClasseHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public AliasClasse existe(Integer classe, Integer alias, Integer orgaoConcedente, Integer aliasClasseId) {
		Criteria c = makeCriteria();

		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		if( classe != null ) {
			c.add(Restrictions.eq("classe.claseservicioId",classe));	
		}
		
		if ( alias != null ) {
			c.add(Restrictions.eq("alias.claseservicioId",alias));	
		}
		
		if( orgaoConcedente != null ) {			
			c.add(Restrictions.eq("orgaoConcedente.orgaoConcedenteId", orgaoConcedente));
		}
		
		if( aliasClasseId != null ) {
			c.add(Restrictions.eq("aliasClasseId",aliasClasseId));	
		}
		
		return (AliasClasse) c.uniqueResult();
	}

	@Override
	public List<AliasClasse> buscar(Integer classe, Integer alias, Integer orgaoConcedente) {
		Criteria c = makeCriteria();
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		if( classe != null ) {
			c.add(Restrictions.eq("classe.claseservicioId",classe));	
		}
		
		if ( alias != null ) {
			c.add(Restrictions.eq("alias.claseservicioId",alias));	
		}
		
		if( orgaoConcedente != null ) {			
			c.add(Restrictions.eq("orgaoConcedente.orgaoConcedenteId", orgaoConcedente));
		}
		
		return c.list();
	}

	@Override
	public List<AliasClasse> buscar(Integer classe, Integer alias, Integer orgaoConcedente, Integer empresaId, Integer rutaId) {
		Criteria c = makeCriteria();
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		if( classe != null ) {
			c.add(Restrictions.eq("classe.claseservicioId",classe));	
		}
		
		if ( alias != null ) {
			c.add(Restrictions.eq("alias.claseservicioId",alias));	
		}
		
		if( orgaoConcedente != null ) {			
			c.add(Restrictions.eq("orgaoConcedente.orgaoConcedenteId", orgaoConcedente));
		}

		c.add(empresaId == null ? Restrictions.isNull("empresa.empresaId") : Restrictions.eq("empresa.empresaId", empresaId));
		c.add(rutaId == null ? Restrictions.isNull("ruta.rutaId") : Restrictions.eq("ruta.rutaId", rutaId));

		return c.list();
	}

}
