package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.AuditarID;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@Table(name = "OPERADOR_EMBARCADA_SERVICO")
@AuditarClasse(nome = "OPERADOR_EMBARCADA_SERVICO", tela = "auditarClasse.OperadorEmbarcadaServico")
@SequenceGenerator(name = "OPERADOR_EMBARCADA_SERVICO_SEQ", sequenceName = "OPERADOR_EMBARCADA_SERVICO_SEQ", allocationSize = 1)
public class OperadorEmbarcadaServico implements Serializable, Auditavel<OperadorEmbarcadaServico> {

	private static final long serialVersionUID = 1L;
	
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "OPERADOR_EMBARCADA_SERVICO_SEQ")
	@Column(name = "OPERADOR_EMBARCADA_SERVICO_ID")
	@AuditarID
	private Long operadorEmbarcadaServicoId;
	
	@ManyToOne
	@JoinColumn(name = "OPERADOR_EMBARCADA_ID", referencedColumnName = "OPERADOR_EMBARCADA_ID")
	private OperadorEmbarcada operadorEmbarcada;
	
	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "ESQUEMA_CORRIDA_ID")
	private EsquemaCorrida corrida;
	
	@Column(name = "ACTIVO")
	private Boolean activo;
	
	@Transient
	@NaoAuditar
	private OperadorEmbarcadaServico operadorClone;

	public Long getOperadorEmbarcadaServicoId() {
		return operadorEmbarcadaServicoId;
	}

	public void setOperadorEmbarcadaServicoId(Long operadorEmbarcadaServicoId) {
		this.operadorEmbarcadaServicoId = operadorEmbarcadaServicoId;
	}

	public OperadorEmbarcada getOperadorEmbarcada() {
		return operadorEmbarcada;
	}

	public void setOperadorEmbarcada(OperadorEmbarcada operadorEmbarcada) {
		this.operadorEmbarcada = operadorEmbarcada;
	}

	public Boolean getActivo() {
		return activo;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public EsquemaCorrida getCorrida() {
		return corrida;
	}

	public void setCorrida(EsquemaCorrida corrida) {
		this.corrida = corrida;
	}

	@Override
	public void clonar() throws CloneNotSupportedException {
		operadorClone = new OperadorEmbarcadaServico();
		operadorClone = (OperadorEmbarcadaServico) this.clone();
	}

	@Override
	public OperadorEmbarcadaServico getCloneObject() throws CloneNotSupportedException {
		return operadorClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getOperadorEmbarcadaServicoId() + "- servico: " + corrida.getNumCorrida() + " - " + corrida.getInfoCorrida());
	}

	@Override
	public String toString() {
		return "" + corrida.getNumCorrida();
	}	
	
	
}
