package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ClienteCursoDAO;
import com.rjconsultores.ventaboletos.entidad.Cliente;
import com.rjconsultores.ventaboletos.entidad.ClienteCurso;
import com.rjconsultores.ventaboletos.entidad.Curso;
import com.rjconsultores.ventaboletos.service.ClienteCursoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("clienteCursoService")
public class ClienteCursoServiceImpl implements ClienteCursoService {

	@Autowired
	private ClienteCursoDAO clienteCursoDAO;

	@Override
	public List<ClienteCurso> obtenerTodos() {
		return clienteCursoDAO.obtenerTodos();
	}

	@Override
	public ClienteCurso obtenerID(Integer id) {
		return clienteCursoDAO.obtenerID(id);
	}

	@Transactional
	@Override
	public ClienteCurso suscribir(ClienteCurso entidad) {

		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return clienteCursoDAO.suscribir(entidad);
	}
	
	@Transactional
	@Override
	public ClienteCurso actualizacion(ClienteCurso entidad) {

		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return clienteCursoDAO.actualizacion(entidad);
	}

	@Transactional
	@Override
	public void borrar(ClienteCurso entidad) {

		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		clienteCursoDAO.actualizacion(entidad);
	}

	@Override
	public ClienteCurso buscarClienteCurso(Cliente cliente, Curso curso) {
		return clienteCursoDAO.buscarClienteCurso(cliente, curso);
	}

}
