package com.rjconsultores.ventaboletos.service.impl;

import java.lang.reflect.Type;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.zkoss.util.resource.Labels;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.rjconsultores.ventaboletos.dao.RevenueConfigDAO;
import com.rjconsultores.ventaboletos.entidad.RevenueConfig;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.service.RevenueConfigService;
import com.rjconsultores.ventaboletos.vo.revenue.RevenueGeoloc;

@Service("revenueConfigService")
public class RevenueConfigServiceImpl implements RevenueConfigService {
	
	private static Logger log = LogManager.getLogger(RevenueConfigService.class);
	
	@Autowired
	private RevenueConfigDAO revenueConfigDAO;

	@Override
	public RevenueConfig obtenerRevenueConfigActivo() {
		return revenueConfigDAO.obtenerRevenueConfigActivo();
	}

	@Override
	public RevenueGeoloc pesquisarGeoloc(String endereco) throws BusinessException {
		RevenueGeoloc retorno = null;
		try {
			RevenueConfig revenueConfig = this.obtenerRevenueConfigActivo();
			verificarRevenueConfigGeoloc(revenueConfig);
			
			HttpPost request = new HttpPost(revenueConfig.getUrlGeoLoc());
			request.addHeader("token-access", revenueConfig.getToken());
			request.addHeader("connector-id", revenueConfig.getConnectorId());
			request.addHeader("Content-type", "application/json");
			
			JSONObject payload = new JSONObject();
			payload.put("endereco", endereco);
			HttpEntity entityPost = new StringEntity(payload.toString(), ContentType.APPLICATION_JSON);
			request.setEntity(entityPost);
			
			CloseableHttpClient client = HttpClientBuilder.create().build();
			HttpResponse response = client.execute(request);
			
			if (response.getStatusLine().getStatusCode() == 200 && response.getEntity() != null) {
				Type listType = new TypeToken<List<RevenueGeoloc>>(){}.getType();
				String entity = EntityUtils.toString(response.getEntity());
				List<RevenueGeoloc> retornos = new Gson().fromJson(entity, listType);
				if (!retornos.isEmpty()) {
					retorno = retornos.get(0);
				}
			}
		} catch (BusinessException e) {
			throw e;
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			throw new BusinessException(e.getMessage());
		}
		return retorno;
	}

	private void verificarRevenueConfigGeoloc(RevenueConfig revenueConfig) throws BusinessException {
		if (revenueConfig == null || StringUtils.isBlank(revenueConfig.getUrlGeoLoc())) {
			throw new BusinessException(Labels.getLabel("editarCatalogoDeParadaController.MSG.RevenueGeolocConfig.value"));
		}
	}

}
